@@ -152,4 +152,65 @@ def test_issue_63():
 
     @with_signature(signature(a))
     def test(*args, **kwargs):
-        return a(*args, **kwargs)
\ No newline at end of file
+        return a(*args, **kwargs)
+
+
+def test_issue_66():
+    """Chain of @wraps with sig mod https://github.com/smarie/python-makefun/issues/66"""
+
+    def a(foo):
+        return foo + 1
+
+    assert a(1) == 2
+
+    # create a first wrapper that is signature-preserving
+
+    @wraps(a)
+    def wrapper(foo):
+        return a(foo) - 1
+
+    assert wrapper(1) == 1
+
+    # the __wrapped__ attr is here:
+    assert wrapper.__wrapped__ is a
+
+    # create a second wrapper that is not signature-preserving
+
+    @wraps(wrapper, append_args="bar")
+    def second_wrapper(foo, bar):
+        return wrapper(foo) + bar
+
+    assert second_wrapper(1, -1) == 0
+
+    with pytest.raises(AttributeError):
+        second_wrapper.__wrapped__
+
+
+def test_issue_pr_67():
+    """Test handcrafted for https://github.com/smarie/python-makefun/pull/67"""
+
+    class CustomException(Exception):
+        pass
+
+    class Foo(object):
+        def __init__(self, a=None):
+            if a is None:
+                raise CustomException()
+
+        def __repr__(self):
+            # this is a valid string but calling eval on it will raise an
+            return "Foo()"
+
+    f = Foo(a=1)
+
+    with pytest.raises(CustomException):
+        eval(repr(f))
+
+    def foo(a=Foo(a=1)):
+        pass
+
+    @wraps(foo, prepend_args="r")
+    def bar(*args, **kwargs):
+        pass
+
+    bar(1)
