@@ -1,3 +1,5 @@
+from itertools import product
+from json import dumps
 import logging
 
 import nox  # noqa
@@ -5,17 +7,20 @@ from pathlib import Path  # noqa
 import sys
 
 # add parent folder to python path so that we can import noxfile_utils.py
-# note that you need to "pip install makefun" for this file to work.
+# note that you need to "pip install -r noxfile-requiterements.txt" for this file to work.
 sys.path.append(str(Path(__file__).parent / "ci_tools"))
 from nox_utils import PY27, PY37, PY36, PY35, PY38, power_session, rm_folder, rm_file, PowerSession  # noqa
 
-ALL_PY_VERSIONS = [PY38, PY37, PY36, PY35, PY27]
+
+pkg_name = "makefun"
+gh_org = "smarie"
+gh_repo = "python-makefun"
 
 ENVS = {
+    PY38: {"coverage": False, "pkg_specs": {"pip": ">19"}},
     PY27: {"coverage": False, "pkg_specs": {"pip": ">10"}},
     PY35: {"coverage": False, "pkg_specs": {"pip": ">10"}},
     PY36: {"coverage": False, "pkg_specs": {"pip": ">19"}},
-    PY38: {"coverage": False, "pkg_specs": {"pip": ">19"}},
     # IMPORTANT: this should be last so that the folder docs/reports is not deleted afterwards
     PY37: {"coverage": True, "pkg_specs": {"pip": ">19"}},  # , "pytest-html": "1.9.0"
 }
@@ -30,7 +35,7 @@ nox.options.default_venv_backend = "conda"
 # nox.options.verbose = True
 
 nox_logger = logging.getLogger("nox")
-nox_logger.setLevel(logging.INFO)
+# nox_logger.setLevel(logging.INFO)  NO !!!! this prevents the "verbose" nox flag to work !
 
 
 class Folders:
@@ -57,31 +62,54 @@ def tests(session: PowerSession, coverage, pkg_specs):
     rm_file(Folders.root / ".coverage")
     rm_file(Folders.root / "coverage.xml")
 
+    # CI-only dependencies
+    # Did we receive a flag through positional arguments ? (nox -s tests -- <flag>)
+    # install_ci_deps = False
+    # if len(session.posargs) == 1:
+    #     assert session.posargs[0] == "keyrings.alt"
+    #     install_ci_deps = True
+    # elif len(session.posargs) > 1:
+    #     raise ValueError("Only a single positional argument is accepted, received: %r" % session.posargs)
+
     # uncomment and edit if you wish to uninstall something without deleting the whole env
-    # session_run(session, "pip uninstall pytest-asyncio --yes")
+    # session.run2("pip uninstall pytest-asyncio --yes")
 
     # install all requirements
     session.install_reqs(setup=True, install=True, tests=True, versions_dct=pkg_specs)
 
+    # install CI-only dependencies
+    # if install_ci_deps:
+    #     session.install2("keyrings.alt")
+
+    # list all (conda list alone does not work correctly on github actions)
+    # session.run2("conda list")
+    conda_prefix = Path(session.bin)
+    if conda_prefix.name == "bin":
+        conda_prefix = conda_prefix.parent
+    session.run2("conda list", env={"CONDA_PREFIX": str(conda_prefix), "CONDA_DEFAULT_ENV": session.get_session_id()})
+
+    # Fail if the assumed python version is not the actual one
+    session.run2("python ci_tools/check_python_version.py %s" % session.python)
+
     # install self so that it is recognized by pytest
     session.run2("pip install -e . --no-deps")
 
     # check that it can be imported even from a different folder
-    session.run2(['python', '-c', '"import os; os.chdir(\'./docs/\'); import makefun"'])
+    session.run2(['python', '-c', '"import os; os.chdir(\'./docs/\'); import %s"' % pkg_name])
 
     # finally run all tests
     if not coverage:
         # simple: pytest only
-        session.run2("python -m pytest -v makefun/tests/")
+        session.run2("python -m pytest -v %s/tests/" % pkg_name)
     else:
         # coverage + junit html reports + badge generation
         session.install_reqs(phase="coverage", phase_reqs=["coverage", "pytest-html", "requests", "xunitparser"],
                              versions_dct=pkg_specs)
 
         # --coverage + junit html reports
-        session.run2("coverage run --source makefun "
-                     "-m pytest --junitxml={dst}/junit.xml --html={dst}/report.html -v makefun/tests/"
-                     "".format(dst=Folders.test_reports))
+        session.run2("coverage run --source {pkg_name} "
+                     "-m pytest --junitxml={dst}/junit.xml --html={dst}/report.html -v {pkg_name}/tests/"
+                     "".format(pkg_name=pkg_name, dst=Folders.test_reports))
         # session.run2("coverage report")  # this shows in terminal + fails under XX%, same as --cov-report term --cov-fail-under=70  # noqa
         session.run2("coverage xml -o {covxml}".format(covxml=Folders.coverage_xml))
         session.run2("coverage html -d {dst}".format(dst=Folders.coverage_reports))
@@ -124,9 +152,9 @@ def publish(session: PowerSession):
 
     # publish the coverage - now in github actions only
     # session.install_reqs(phase="codecov", phase_reqs=["codecov", "keyring"])
-    # # keyring set https://app.codecov.io/gh/smarie/python-makefun token
+    # # keyring set https://app.codecov.io/gh/<org>/<repo> token
     # import keyring  # (note that this import is not from the session env but the main nox env)
-    # codecov_token = keyring.get_password("https://app.codecov.io/gh/smarie/python-makefun", "token")
+    # codecov_token = keyring.get_password("https://app.codecov.io/gh/<org>/<repo>>", "token")
     # # note: do not use --root nor -f ! otherwise "There was an error processing coverage reports"
     # session.run2('codecov -t %s -f %s' % (codecov_token, Folders.coverage_xml))
 
@@ -181,9 +209,35 @@ def release(session: PowerSession):
     # create the github release
     session.install_reqs(phase="release", phase_reqs=["click", "PyGithub"])
     session.run2("python ci_tools/github_release.py -s {gh_token} "
-                 "--repo-slug smarie/python-makefun -cf ./docs/changelog.md "
-                 "-d https://smarie.github.io/python-makefun/changelog/ {tag}"
-                 "".format(gh_token=gh_token, tag=current_tag))
+                 "--repo-slug {gh_org}/{gh_repo} -cf ./docs/changelog.md "
+                 "-d https://{gh_org}.github.io/{gh_repo}/changelog/ {tag}"
+                 "".format(gh_token=gh_token, gh_org=gh_org, gh_repo=gh_repo, tag=current_tag))
+
+
+@nox.session(python=False)
+def gha_list(session):
+    """(mandatory arg: <base_session_name>) Prints all sessions available for <base_session_name>, for GithubActions."""
+
+    # see https://stackoverflow.com/q/66747359/7262247
+
+    # get the desired base session to generate the list for
+    if len(session.posargs) != 1:
+        raise ValueError("This session has a mandatory argument: <base_session_name>")
+    session_func = globals()[session.posargs[0]]
+
+    # list all sessions for this base session
+    try:
+        session_func.parametrize
+    except AttributeError:
+        sessions_list = ["%s-%s" % (session_func.__name__, py) for py in session_func.python]
+    else:
+        sessions_list = ["%s-%s(%s)" % (session_func.__name__, py, param)
+                         for py, param in product(session_func.python, session_func.parametrize)]
+
+    # print the list so that it can be caught by GHA.
+    # Note that json.dumps is optional since this is a list of string.
+    # However it is to remind us that GHA expects a well-formatted json list of strings.
+    print(dumps(sessions_list))
 
 
 # if __name__ == '__main__':
